# frozen_string_literal: true

require "is/inspectable"

module Core
  module Watch
    # [public] Callback for a watched system.
    #
    class Callback
      class << self
        # [public] Build a callback instance for a given value and block.
        #
        def build(value, &block)
          case value
          when Callback
            value
          when ::NilClass
            Callback.new(&block)
          when ::Regexp
            Callbacks::Regexp.new(value, &block)
          else
            Callbacks::Path.new(value, &block)
          end
        end
      end

      include Is::Inspectable

      def initialize(&block)
        @block = block
      end

      # [public] Invoke the callback.
      #
      def call(*args, **kwargs)
        @block.call(*args, **kwargs)
      end

      # [public]
      #
      def match?(_)
        true
      end
    end

    require_relative "callbacks/path"
    require_relative "callbacks/regexp"
  end
end
