# frozen_string_literal: true

require "is/inspectable"

module Core
  module Watch
    # [public] The status of a system watch.
    #
    class Status
      include Is::Inspectable
      inspects without: [:@mutex]

      STATES = %i[running paused stopped].freeze

      def initialize
        @state = :stopped
        @mutex = Mutex.new
      end

      STATES.each do |state|
        # [public]
        #
        class_eval <<~CODE, __FILE__, __LINE__ + 1
          def #{state}!
            @mutex.synchronize {
              if @state == #{state.inspect}
                false
              else
                @state = #{state.inspect}
                true
              end
            }
          end
        CODE

        # [public]
        #
        class_eval <<~CODE, __FILE__, __LINE__ + 1
          def #{state}?
            @state == #{state.inspect} || (#{state.inspect} == :running && @state == :paused)
          end
        CODE
      end
    end
  end
end
