# frozen_string_literal: true

require "is/inspectable"
require "openssl"

require_relative "../strategy"

module Core
  module Watch
    module Strategies
      # [public] Identifies a path by digest or modified time (for directories).
      #
      class Digest
        include Is::Inspectable

        BUFFER_LENGTH = 16_384

        # [public]
        #
        def identify(path)
          if path.file?
            md5 = OpenSSL::Digest.new("MD5")

            path.open("rb") do |file|
              buffer = +""
              while file.read(BUFFER_LENGTH, buffer)
                md5 << buffer
              end
            end

            md5.digest
          else
            path.mtime
          end
        end
      end
    end
  end
end
